#include "http_get.h"

#include <stdio.h>
#include <tchar.h>
#include <string.h>

namespace webdav
{

HttpGet::HttpGet()
{
}

HttpGet::~HttpGet()
{
}

/* Dispatch a GET request REQ, writing the response body to FD fd.  If
 * RANGE is non-NULL, then it is the value of the Range request
 * header, e.g. "bytes=1-5".  Returns an NE_* error code. */
int HttpGet::dispatch_to_fd(ne_request *req, int fd, const char *range)
{
    ne_session *const sess = ne_get_session(req);
    const ne_status *const st = ne_get_status(req);
    int ret;
    size_t rlen;

    /* length of bytespec after "bytes=" */
    rlen = range ? strlen(range + 6) : 0;

    do {
        const char *value;
        
        ret = ne_begin_request(req);
        if (ret != NE_OK) break;

        value = ne_get_response_header(req, "Content-Range");

        /* For a 206 response, check that a Content-Range header is
         * given which matches the Range request header. */
        if (range && st->code == 206 
            && (value == NULL || strncmp(value, "bytes ", 6) != 0
                || strncmp(range + 6, value + 6, rlen)
                || value[6 + rlen] != '/')) {
            // ne_set_error(sess, _("Response did not include requested range"));
            return NE_ERROR;
        }

        if ((range && st->code == 206) || (!range && st->klass == 2)) {
            ret = ne_read_response_to_fd(req, fd);
        } else {
            ret = ne_discard_response(req);
        }

        if (ret == NE_OK) ret = ne_end_request(req);
    } while (ret == NE_RETRY);

    return ret;
}

bool HttpGet::Get(ne_session* sess, const char* rpath, const char* lpath)
{
	FILE* fp = _tfopen(lpath, _T("wb"));
	if(fp == NULL){
		// TODO: error handling
		return false;
	}

	const char *uri = rpath;
	int fd = _fileno(fp);

    ne_request *req = ne_request_create(sess, "GET", uri);

	ne_add_response_body_reader(req, AcceptResponse, response_handler_, response_handler_data_);
	ne_set_request_body_provider(req, 0, request_handler_, request_handler_data_);

    int ret = dispatch_to_fd(req, fd, NULL);

    if(ret == NE_OK && ne_get_status(req)->klass != 2){
		ret = NE_ERROR;
    }

    ne_request_destroy(req);

	fclose(fp);

	if(ret != NE_OK){
		return false;
	}

	return true;
}

}
