#include <ne_auth.h>
#include <ne_basic.h>
#include "session.h"

#define CONNECT_TIMEOUT 10
#define READ_TIMEOUT 10

namespace webdav
{

Session::Session()
	: sess_(NULL)
	, is_connected_(false)
{
}

Session::~Session()
{
	Disconnect();
}

void Session::Connect(const char* scheme,
					  const char* hostname,
					  unsigned int port,
					  ne_progress notify_progress_func,
					  ne_notify_status notify_status_func,
					  ne_ssl_verify_fn ssl_verify_fn,
					  WebDAV* webdav)
{
	bool use_ssl;

	use_ssl = false;

	Disconnect();

    if(stricmp(scheme, "https") == 0){
        if(!ne_has_support(NE_FEATURE_SSL)) {
			// neonhttpsT|[gŃRpCĂ
			sess_ = NULL;
            return;
        }

        use_ssl = true;
    }

	sess_ = ne_session_create(scheme, hostname, port);
	if(sess_ == NULL){
		// TODO: error
		return;
	}

	ne_set_progress(sess_, notify_progress_func, webdav);
	//ne_set_notifier(sess_, notify_status_func, webdav);
	ne_set_connect_timeout(sess_, CONNECT_TIMEOUT);
	ne_set_read_timeout(sess_, READ_TIMEOUT);

	if(use_ssl){
		// SSL
		ne_ssl_trust_default_ca(sess_);
		ne_ssl_set_verify(sess_, ssl_verify_fn, webdav);
	}

	// authorization
	ne_set_server_auth(sess_, SupplyCredsServer, this);
	ne_set_proxy_auth(sess_, SupplyCredsProxy, this);

	if(proxy_host_.length() > 0){
		ne_session_proxy(sess_, proxy_host_.c_str(), proxy_port_);
	}

	is_connected_ = true;

	return;
}

void Session::Connect(const char* scheme,
					  const char* hostname,
					  ne_progress notify_progress_func,
					  ne_notify_status notify_status_func,
					  ne_ssl_verify_fn ssl_verify_fn,
					  WebDAV* webdav)
{
	Connect(scheme, hostname, URI::GetDefaultPort(scheme), notify_progress_func, notify_status_func, ssl_verify_fn, webdav);
}

bool Session::IsConnected() const
{
	return is_connected_;
}

void Session::Disconnect()
{
	if(sess_ == NULL){
		return;
	}

	ne_session_destroy(sess_);
	sess_ = NULL;
	is_connected_ = false;
}

int Session::SupplyCredsServer(void *userdata, const char *realm, int attempt,
							   char *username, char *password)
{
	strncpy(username, ((Session*)userdata)->username_.c_str(), NE_ABUFSIZ);
	strncpy(password, ((Session*)userdata)->password_.c_str(), NE_ABUFSIZ);

	return attempt;
}

int Session::SupplyCredsProxy(void *userdata, const char *realm, int attempt,
							  char *username, char *password)
{
	strncpy(username, ((Session*)userdata)->proxy_username_.c_str(), NE_ABUFSIZ);
	strncpy(password, ((Session*)userdata)->proxy_password_.c_str(), NE_ABUFSIZ);

	return attempt;
}

const char* Session::GetError() const
{
	return ne_get_error(sess_);
}

URI Session::GetURI()
{
	if(sess_ == NULL){
		URI uri;
		return uri;
	}

	ne_uri buf = {0};

	ne_fill_server_uri(sess_, &buf);

	return buf;
}

std::string Session::GetScheme() const
{
	return ne_get_scheme(sess_);
}

} // namespace webdav
