
#include "uri.h"
#include <ne_alloc.h>
#include <tchar.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

namespace webdav
{

URI::URI()
{
	ne_uri uri;
	memset(&uri, 0, sizeof(uri));
	URI::URI(uri);
}

URI::URI(ne_uri uri)
{
	uri_ = uri;

	if(!uri_.scheme){
		uri_.scheme = _tcsdup(_T(""));
	}

	if(!uri_.host){
		uri_.host = _tcsdup(_T(""));
	}

	if(!uri_.path){
		uri_.path = _tcsdup(_T(""));
	}

	if(!uri_.userinfo){
		uri_.userinfo = _tcsdup(_T(""));
	}
}

URI::URI(const char* uri)
{
	ne_uri_parse(uri, &uri_);
}

URI::~URI()
{
	ne_uri_free(&uri_);
}

URI& URI::operator =(const URI& other)
{
	if(this == &other){
		return *this;
	}

	return *this;
}

URI& URI::operator =(const char* other)
{
	if(other == NULL){
		memset((void*)&uri_, 0, sizeof(uri_));
	}

	ne_uri_parse(other, &uri_);

	return *this;
}

std::string URI::GetScheme() const
{
	if(uri_.scheme == NULL){
		return _T("");
	}

	return uri_.scheme;
}

std::string URI::GetHost() const
{
	if(uri_.host == NULL){
		return _T("");
	}

	return uri_.host;
}

unsigned int URI::GetPort() const
{
	return uri_.port;
}

std::string URI::GetPath() const
{
	if(uri_.path == NULL){
		return _T("");
	}

	return uri_.path;
}

std::string URI::GetAuthInfo() const
{
	return uri_.userinfo;
}

void URI::SetScheme(const char* scheme)
{
	if(uri_.scheme){
		free(uri_.scheme);
	}

	size_t size = strlen(scheme) + 1;
	uri_.scheme = (char*)malloc(size);
	strncpy(uri_.scheme, scheme, size);
}

void URI::SetHost(const char* host)
{
	if(uri_.host){
		free(uri_.host);
	}

	size_t size = strlen(host) + 1;
	uri_.host = (TCHAR*)malloc(size);
	strncpy(uri_.host, host, size);
}

void URI::SetPort(unsigned int port)
{
	uri_.port = port;
}

void URI::SetPath(const char* path)
{
	if(uri_.path){
		free(uri_.path);
	}

	size_t size = strlen(path) + 1;
	uri_.path = (TCHAR*)malloc(size);
	strncpy(uri_.path, path, size);
}

void URI::SetAuthInfo(const char* userinfo)
{
	if(uri_.userinfo){
		free(uri_.userinfo);
	}

	size_t size = strlen(userinfo) + 1;
	uri_.userinfo = (TCHAR*)malloc(size);
	strncpy(uri_.userinfo, userinfo, size);
}

unsigned int URI::GetDefaultPort(const char* scheme)
{
	return ne_uri_defaultport(scheme);
}

} // namespace WebDAV
