#include "utils_impl.h"
#include "uri.h"
#include "ne_string.h"
#include "ne_uri.h"

namespace webdav
{

UtilsImpl::UtilsImpl()
{
}

UtilsImpl::~UtilsImpl()
{
}

const char* UtilsImpl::GetParentPath(const char* path)
{
	char* tmp = ne_path_parent(path);
	if(tmp == NULL){
		return path;
	}

	std::string parent(tmp);
	if(tmp != NULL){
		free(tmp);
	}

	tmp_ = parent;

	return tmp_.c_str();
}

const char* UtilsImpl::GetPath(const char* url)
{
	URI uri(url);

	tmp_ = uri.GetPath();

	return tmp_.c_str();
}

const char* UtilsImpl::ResolvePath(const char* p, const char* filename, bool iscoll)
{
	char* ret = NULL;
	const char* pnt = NULL;

	pnt = Escape(filename);
	ret = ne_concat(p, pnt, NULL);

	if(ret != NULL){
		tmp_ = ret;
		free(ret);
	}

	return tmp_.c_str();
}

const char* UtilsImpl::GetBaseName(const char* path2)
{
	std::string path(path2);
	std::string name;

	if(ne_path_has_trailing_slash(path.c_str())) {
		path[strlen(path.c_str()) - 1] = '\0';
	}

	const char* tmp = strrchr(path.c_str(), '/');
	if(tmp == NULL){
		name = path;
	}else{
		name = tmp + 1;
	}

	tmp_ = Unescape(name.c_str());

	return tmp_.c_str();
}

const char* UtilsImpl::GetBaseName(const Resource& res)
{
	return GetBaseName(res.GetPath());
}

const char* UtilsImpl::Escape(const char* src)
{
	char* tmp = ne_path_escape(src);

	if(tmp == NULL){
		tmp_ = "";
		return tmp_.c_str();
	}

	tmp_ = tmp;
	free((void*)tmp);

	return tmp_.c_str();
}

const char* UtilsImpl::Unescape(const char* p)
{
	char* tmp = ne_path_unescape(p);
	if(tmp == NULL){
		tmp_ = "";
		return tmp_.c_str();
	}

	tmp_ = tmp;
	free((void*)tmp);

	return tmp_.c_str();
}

}
